/* Copyright 2013 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Rajit Shahi
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 */

package magma.tools.logfile;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import magma.util.connection.IServerConnection;
import magma.util.connection.impl.RCSSLogFileConnection;
import magma.util.file.TarBz2ZipUtil;

/**
 * Command line tool to cut off cycles from logfiles before game start and after
 * game end
 * @author kdorer
 */
public class LogfileCutter
{
	/** true if start and end should be cut */
	private boolean cut;

	/** true if digits from floats should be chopped off */
	private boolean shrink;

	public static void main(String[] args)
	{
		if (args.length < 1) {
			System.out
					.println("Usage: magma.tools.logfile.LogFileCutter <file> [bz2|gz|uncompressed] [cut|shrink|both]");
			System.exit(-1);
		}

		// create name of the output file
		String outFileName = getOutFilename(args);
		boolean cut = getCutArgument(args);
		boolean shrink = getShrinkArgument(args);

		LogfileCutter cutter = new LogfileCutter(cut, shrink);

		File inFile = new File(args[0]);
		File outFile = new File(outFileName);
		// System.out.println("Cutting file: " + inFile);
		System.out.println("Into file: " + outFile);
		cutter.processFile(inFile, outFile);
	}

	/**
	 * @param args
	 * @return
	 */
	private static boolean getShrinkArgument(String[] args)
	{
		if (args.length < 3) {
			// default is shrink
			return true;
		}
		if ("shrink".equalsIgnoreCase(args[2])
				|| "both".equalsIgnoreCase(args[2])) {
			return true;
		}
		return false;
	}

	/**
	 * @param args
	 * @return
	 */
	private static boolean getCutArgument(String[] args)
	{
		if (args.length < 3) {
			// default is cut
			return true;
		}
		if ("cut".equalsIgnoreCase(args[2]) || "both".equalsIgnoreCase(args[2])) {
			return true;
		}
		return false;
	}

	/**
	 * @param args
	 * @return
	 */
	public static String getOutFilename(String[] args)
	{
		String outFileName = args[0].replaceFirst("\\.", "_cut.");
		String outExtension = "bz2";
		if (args.length > 1) {
			outExtension = args[1];
		}

		String inType = "\\.bz2";
		if (args[0].endsWith("tar.bz2")) {
			inType = "\\.tar.bz2";
		} else if (args[0].endsWith("gz")) {
			inType = "\\.gz";
		}

		if (outExtension.equalsIgnoreCase("uncompressed")) {
			outFileName = args[0].replaceFirst(inType, "_cut.log");
		} else if (outExtension.equalsIgnoreCase("gz")) {
			outFileName = args[0].replaceFirst(inType, "_cut.gz");
		} else if (outExtension.equalsIgnoreCase("bz2")) {
			outFileName = args[0].replaceFirst(inType, "_cut.bz2");
		} else {
			System.out.println("Invalid output file type: " + outExtension);
			System.exit(-1);
		}
		return outFileName;
	}

	/**
	 * @param cut2
	 * @param shrink2
	 */
	public LogfileCutter(boolean cut, boolean shrink)
	{
		this.cut = cut;
		this.shrink = shrink;
	}

	/**
	 * Cuts off all cycles before game start and after game end of the logfile
	 * passed
	 * @param inFile
	 * @param outFile
	 */
	public void processFile(File inFile, File outFile)
	{
		boolean skipping = true;
		String lastFieldMessage = null;
		String message = null;
		String lastMessage = null;
		float time = 0;
		float firstTime = Float.NEGATIVE_INFINITY;
		PrintWriter out = null;

		try {
			out = TarBz2ZipUtil.createPrintWriter(outFile);

			IServerConnection connection = new RCSSLogFileConnection(inFile);
			connection.establishConnection();

			while (connection.next()) {
				lastMessage = message;
				message = connection.getCurrentMessage();

				if (cut) {
					// remember last full simulation frame
					if (message.contains("(FieldLength")) {
						lastFieldMessage = message;
					}

					// get current game time
					time = getCurrentGameTime(message);
					if (firstTime == Float.NEGATIVE_INFINITY) {
						firstTime = time;
					}
					if (time > firstTime) {
						// after game start
						if (skipping) {
							// write header
							// if (lastFieldMessage != lastMessage) {
							writeMessage(lastFieldMessage, out);
							writeMessage(lastMessage, out);
							writeMessage(message, out);
							skipping = false;

						} else {
							if ((firstTime <= 0 && time >= 300) //
									|| (firstTime >= 300 && time >= 600)) {
								writeMessage(message, out);
								break;
							}

							writeMessage(message, out);
						}
					}
				} else {
					// we do not cut the logfile
					writeMessage(message, out);
				}
			}

			connection.stopReceiveLoop();
			out.close();

		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

	}

	/**
	 * @param message
	 * @param out
	 */
	public void writeMessage(String message, PrintWriter out)
	{
		String newMessage = message;
		if (shrink) {
			// replace all 6 digit precision floating points with 3 digits
			String regex = "(\\.)([0-9]{3})([0-9]{3})";
			newMessage = message.replaceAll(regex, "$1$2");
		}

		out.println(newMessage);
	}

	/**
	 * @param message
	 * @return
	 */
	private float getCurrentGameTime(String message)
	{
		int startIndex = message.indexOf("(time ");
		if (startIndex >= 0) {
			startIndex += 6;
			int endIndex = message.substring(startIndex).indexOf(')');
			return Float.parseFloat(message.substring(startIndex, startIndex
					+ endIndex));
		}
		return -1;
	}
}
